using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using iNTERFACEWARE.Chameleon;
using PrefixTest;


/// <summary>
/// This application is a simple example of using the Chameleon generated objects in a 'Client'
/// configuration.  It is intended to be a helpful starting point in understanding how to use 
/// these objects, but must be fleshed out in the following areas to provide a full solution for an
/// application HL7 interface:
/// 1) You will need to work out how your application will handle the logging of messages
///    and data, in particular how bad transactions should be logged for effective troubleshooting.
/// 2) The code will need to be modified to insert the parsed data into a database, whether
///    by using explicit database calls or by going through some type of middle tier layer that
///    your application may support.
/// 3) Some customers choose to deploy HL7 interfaces as Windows Services.  This can be done
///    easily with Chameleon.  One thing to keep in mind though is that the networking components
///    supplied with Chameleon require a Windows 'message loop'.  Often Windows Services do
///    not have a message loop - but one can be easily set up by invoking RunMessageLoop on
///    the networking component supplied with Chameleon.  See the manual for running Chameleon
///    within services.
/// 4) Definitely take the time to read the manual and go through the tutorials after looking
///    at this example and before attempting to deploy a production solution.  We do provide custom
///    on-site training and technical implementation services if you require them.
///
/// If you are looking for a much more complete 'out of the box' solution for your HL7 needs it 
/// would be a good idea to evaluate iNTERFACEWARE Iguana.  Iguana is powered by Chameleon, but
/// provides a complete run-time web-based infrastructure for your HL7 interfaces.
/// It provides advanced logging and monitoring capability which makes it a very attractive 
/// option for many customers.
/// 
/// It should be strongly emphasized that this example program is only for demonstration purposes.
/// A real HL7 interface should have some type of system of logging into log files.  Iguana, for
/// instance, has a very nice logging system that allows administrators to drill into and browse
/// processed messages and errors.  How you implement this logging is up to your requirements
/// for run-time management of your HL7 interface.
/// </summary>

public class ClientForm : System.Windows.Forms.Form, Log
{
	private System.ComponentModel.Container components = null;
	private ClientEngine engine;
	private string MESSAGE_DEFINITION_FILE = "test.vmd";
	private const int DEFAULT_PORT = 5123;
	private const int MAX_BUFFER = 30000;
	private ArrayList messageGeneratorVector;
	// The llpClient is a .NET component which you can find in the iNTERFACEWARE tab in the ToolBox.
	private iNTERFACEWARE.Chameleon.LlpClient llpClient;
	private bool connected;

	private System.Windows.Forms.Panel _middlePanel;
	private System.Windows.Forms.Label _configurationLabel;
	private System.Windows.Forms.Button _sendButton;
	private System.Windows.Forms.TextBox _logTextArea;
	private System.Windows.Forms.Panel _bottomPanel;
	private System.Windows.Forms.Panel _topPanel;
	private System.Windows.Forms.Label _portLabel;
	private System.Windows.Forms.TextBox _portTextBox;
	private System.Windows.Forms.Label _hostLabel;
	private System.Windows.Forms.TextBox _hostTextBox;
	private System.Windows.Forms.ComboBox _messageCmbBox;
	private System.Windows.Forms.ComboBox _configurationCmbBox;
	private System.Windows.Forms.Button _connectDisconnectBtn;

	static public void Main(String[] args)
	{
		ClientForm FormMain = new ClientForm();
		FormMain.ShowDialog();
	}

	public ClientForm()
	{
		// Required for Windows Form Designer support
		InitializeComponent();
	}

	private void ClientForm_Load(object sender, System.EventArgs e)
	{
		// Initialize client engine
		try
		{
			// In order to make the engine useable we must load the vmd file.  This file
			// contains the information about the structure of the messages, tables, and data
			// mappings that were defined in the Chameleon GUI.  ClientEngine() is a generated 
			// Engine which has an interface based on the messages and tables defined in the vmd file.
			// 
			// Also keep in mind that this example is using the static stubcode 
			// interface of Chameleon. For advanced applications it is possible to 
			// use a dynamic interface to populate messages.  See the manual for 
			// more details.
			engine = new ClientEngine();
			engine.AddLog(this);
			engine.Load(MESSAGE_DEFINITION_FILE);
			Println("Loaded " + MESSAGE_DEFINITION_FILE + "\r\n");
		}
		catch (ChameleonException ex)
		{
			Println(ex.ToString() + " while trying to initialize client engine");
		}

		try
		{ 
			// This example contains a combo box from which the user can select the message
			// to generate.  Keep in mind that this functionality is for teaching purposes
			// only, and that a real HL7 client would most likely be part of a backend service.
			messageGeneratorVector = new ArrayList();
			// Beginning of message generator block (comment used as an anchor for STC)
			messageGeneratorVector.Add(new GeneratePatientRegister());
			messageGeneratorVector.Add(new GenerateDefault());
			// End of message generator block (comment used as an anchor for STC)

			_messageCmbBox.Sorted = false;
			for (int MessageIndex = 0;
				MessageIndex < messageGeneratorVector.Count;
				++MessageIndex)
			{
				_messageCmbBox.Items.Add(((GenerateMessage)messageGeneratorVector[MessageIndex]).Description());
			}
			_messageCmbBox.SelectedIndex=0;
		}
		catch (ChameleonException ex)
		{
			Println(ex.ToString() + " while trying to build message generation list");
		}

		// This example also includes a combo box that allows the user to switch between configurations.
		// For more information regarding multiple configurations, please see the Chameleon manual.
		try
		{
			for (int ConfigurationIndex = 0; 
				ConfigurationIndex <= engine.CountOfConfig() - 1; 
				++ConfigurationIndex)
			{
				_configurationCmbBox.Items.Add(engine.ConfigName(ConfigurationIndex));
			}
			_configurationCmbBox.SelectedIndex = 0;
		}
		catch (ChameleonException ex)
		{
			Println(ex.ToString() + " while trying to build configuration list");
		}

		// The Client is initially not connected to any host
		connected = false;
	}

	private void SelectedIndexChanged(object sender, System.EventArgs e)
	{
		// Change the current engine configuration to the selected configuration
		engine.CurrentConfigIndex = _configurationCmbBox.SelectedIndex;
		Println("Configuration set to " + engine.ConfigName(engine.CurrentConfigIndex) + "\r\n");
	}

	// This method is invoked when the 'Send' button is clicked.  It generates a message
	// and sends it.
	private void SendButtonClicked(object sender, System.EventArgs e)
	{
		string messageData;
		try
		{
			messageData = ((GenerateMessage)messageGeneratorVector[_messageCmbBox.SelectedIndex]).GenerateMessage(engine);
			Println("Sending message:");
			// We use the MakeMessagePrintable function to 'pretty print' the HL7 message 
			// into a format that can be easily read in the log files.  This function converts 
			// carriage return (\r) characters into the string given in the second argument of 
			// MakeMessagePrintable.  In this case it is converted into a carriage return/line feed
			// (\r\n) sequence which displays nicely in the memo control.
			// 
			// The third argument is the maximum length of the message before
			// we truncate it.  This is done because displaying data within a memo
			// control is actually very slow - so displaying a message of 100k in
			// the memo control window can give the appearance of hanging the server
			// (i.e. a long blocking event).
			//
			// Having the third argument set to -1 means we are not truncating the output.
			Println(llpClient.MakeMessagePrintable(messageData,"\r\n",-1));
			llpClient.SendMessage(messageData); 
		}
		catch (ChameleonException ex)
		{
			Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to generate and send message"));
		} 
	}

	// This method describes what happens when the disconnect button is clicked.
	private void ConnectDisconnectButtonClicked(object sender, System.EventArgs e)
	{
		if (connected)
		{
			// Disconnect client from host
			try
			{
				llpClient.Disconnect();
				connected = false;
				Println("Disconnected from host\r\n");
			}
			catch (ChameleonException ex)
			{
				Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to disconnect from host"));
			}
		}
		else
		{
			ushort port;
			try 
			{
				port = System.Convert.ToUInt16(_portTextBox.Text);
			}
			catch
			{
				Println("Invalid port number entered - Default port " + DEFAULT_PORT + " used\r\n");
				port = DEFAULT_PORT;
				_portTextBox.Text = port.ToString();
			}
	
			// Connect to host on port specified above
			try
			{
				llpClient.Connect(_hostTextBox.Text, port);
			}
			catch (ChameleonException ex)
			{
				Println(engine.MakeMessagePrintable(ex.ToString() + " while trying to connect to host"));
			}
		}
	}

	// Log interface
	// This code is only for demonstration purposes.  Because memo controls can only 
	// take a certain amount of data - 32k at the time this example was written - we 
	// check the length of the text and truncate it if it reaches this limit.
	public void Println(string line)
	{
		if(_logTextArea.Text.Length > MAX_BUFFER )
		{
			_logTextArea.Text = _logTextArea.Text.Substring(_logTextArea.Text.Length/2,_logTextArea.Text.Length/2);
		}
		_logTextArea.AppendText(line + "\r\n");
	}
	
	/// <summary>
	/// Clean up any resources being used.
	/// </summary>
	protected override void Dispose(bool disposing)
	{
		if (disposing)
		{
			if (components != null)
			{
				components.Dispose();
			}
		}
		base.Dispose(disposing);
	}

	private void ClientForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
	{
		if(connected)
		{
			llpClient.Disconnect();
			connected = false;
		}
		//we need to call dispose manually for llpClient, otherwise an exception will be thrown
		//by ::destroyWindow 
		llpClient.Dispose();
	}
#region Windows Form Designer generated code
	/// <summary>
	/// Required method for Designer support - do not modify
	/// the contents of this method with the code editor.
	/// </summary>
	private void InitializeComponent()
	{
	   this._bottomPanel = new System.Windows.Forms.Panel();
	   this._portLabel = new System.Windows.Forms.Label();
	   this._portTextBox = new System.Windows.Forms.TextBox();
	   this._hostLabel = new System.Windows.Forms.Label();
	   this._hostTextBox = new System.Windows.Forms.TextBox();
	   this._sendButton = new System.Windows.Forms.Button();
	   this._connectDisconnectBtn = new System.Windows.Forms.Button();
	   this._messageCmbBox = new System.Windows.Forms.ComboBox();
	   this._middlePanel = new System.Windows.Forms.Panel();
	   this._logTextArea = new System.Windows.Forms.TextBox();
	   this._configurationLabel = new System.Windows.Forms.Label();
	   this._configurationCmbBox = new System.Windows.Forms.ComboBox();
	   this.llpClient = new iNTERFACEWARE.Chameleon.LlpClient();
	   this._topPanel = new System.Windows.Forms.Panel();
	   this._bottomPanel.SuspendLayout();
	   this._middlePanel.SuspendLayout();
	   this._topPanel.SuspendLayout();
	   this.SuspendLayout();
	   // 
	   // _bottomPanel
	   // 
	   this._bottomPanel.Controls.AddRange(new System.Windows.Forms.Control[] {
																				  this._portLabel,
																				  this._portTextBox,
																				  this._hostLabel,
																				  this._hostTextBox,
																				  this._sendButton,
																				  this._connectDisconnectBtn,
																				  this._messageCmbBox});
	   this._bottomPanel.Dock = System.Windows.Forms.DockStyle.Bottom;
	   this._bottomPanel.Location = new System.Drawing.Point(0, 296);
	   this._bottomPanel.Name = "_bottomPanel";
	   this._bottomPanel.Size = new System.Drawing.Size(542, 73);
	   this._bottomPanel.TabIndex = 2;
	   // 
	   // _portLabel
	   // 
	   this._portLabel.Location = new System.Drawing.Point(350, 10);
	   this._portLabel.Name = "_portLabel";
	   this._portLabel.Size = new System.Drawing.Size(35, 24);
	   this._portLabel.TabIndex = 0;
	   this._portLabel.Text = "Port:";
	   this._portLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
	   // 
	   // _portTextBox
	   // 
	   this._portTextBox.Location = new System.Drawing.Point(395, 10);
	   this._portTextBox.MaxLength = 5;
	   this._portTextBox.Name = "_portTextBox";
	   this._portTextBox.Size = new System.Drawing.Size(125, 20);
	   this._portTextBox.TabIndex = 1;
	   this._portTextBox.Text = "5123";
	   // 
	   // _hostLabel
	   // 
	   this._hostLabel.Location = new System.Drawing.Point(350, 45);
	   this._hostLabel.Name = "_hostLabel";
	   this._hostLabel.Size = new System.Drawing.Size(35, 24);
	   this._hostLabel.TabIndex = 2;
	   this._hostLabel.Text = "Host:";
	   this._hostLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
	   // 
	   // _hostTextBox
	   // 
	   this._hostTextBox.Location = new System.Drawing.Point(395, 45);
	   this._hostTextBox.Name = "_hostTextBox";
	   this._hostTextBox.Size = new System.Drawing.Size(125, 20);
	   this._hostTextBox.TabIndex = 3;
	   this._hostTextBox.Text = "localhost";
	   // 
	   // _sendButton
	   // 
	   this._sendButton.Enabled = false;
	   this._sendButton.Location = new System.Drawing.Point(12, 42);
	   this._sendButton.Name = "_sendButton";
	   this._sendButton.Size = new System.Drawing.Size(120, 24);
	   this._sendButton.TabIndex = 4;
	   this._sendButton.Text = "&Send";
	   this._sendButton.Click += new System.EventHandler(this.SendButtonClicked);
	   // 
	   // _connectDisconnectBtn
	   // 
	   this._connectDisconnectBtn.Location = new System.Drawing.Point(172, 42);
	   this._connectDisconnectBtn.Name = "_connectDisconnectBtn";
	   this._connectDisconnectBtn.Size = new System.Drawing.Size(120, 24);
	   this._connectDisconnectBtn.TabIndex = 5;
	   this._connectDisconnectBtn.Text = "&Connect";
	   this._connectDisconnectBtn.Click += new System.EventHandler(this.ConnectDisconnectButtonClicked);
	   // 
	   // _messageCmbBox
	   // 
	   this._messageCmbBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
	   this._messageCmbBox.Location = new System.Drawing.Point(12, 12);
	   this._messageCmbBox.Name = "_messageCmbBox";
	   this._messageCmbBox.Size = new System.Drawing.Size(280, 21);
	   this._messageCmbBox.Sorted = true;
	   this._messageCmbBox.TabIndex = 6;
	   // 
	   // _middlePanel
	   // 
	   this._middlePanel.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
		   | System.Windows.Forms.AnchorStyles.Left) 
		   | System.Windows.Forms.AnchorStyles.Right);
	   this._middlePanel.Controls.AddRange(new System.Windows.Forms.Control[] {
																				 this._logTextArea});
	   this._middlePanel.Location = new System.Drawing.Point(0, 45);
	   this._middlePanel.Name = "_middlePanel";
	   this._middlePanel.Size = new System.Drawing.Size(542, 253);
	   this._middlePanel.TabIndex = 1;
	   // 
	   // _logTextArea
	   // 
	   this._logTextArea.BackColor = System.Drawing.SystemColors.Window;
	   this._logTextArea.Cursor = System.Windows.Forms.Cursors.Default;
	   this._logTextArea.Dock = System.Windows.Forms.DockStyle.Fill;
	   this._logTextArea.Font = new System.Drawing.Font("Courier New", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
	   this._logTextArea.ForeColor = System.Drawing.SystemColors.WindowText;
	   this._logTextArea.MaxLength = 0;
	   this._logTextArea.Multiline = true;
	   this._logTextArea.Name = "_logTextArea";
	   this._logTextArea.ReadOnly = true;
	   this._logTextArea.ScrollBars = System.Windows.Forms.ScrollBars.Both;
	   this._logTextArea.Size = new System.Drawing.Size(542, 253);
	   this._logTextArea.TabIndex = 0;
	   this._logTextArea.TabStop = false;
	   this._logTextArea.Text = "";
	   this._logTextArea.WordWrap = false;
	   // 
	   // _configurationLabel
	   // 
	   this._configurationLabel.Location = new System.Drawing.Point(35, 12);
	   this._configurationLabel.Name = "_configurationLabel";
	   this._configurationLabel.Size = new System.Drawing.Size(125, 24);
	   this._configurationLabel.TabIndex = 0;
	   this._configurationLabel.Text = "Current Configuration:";
	   this._configurationLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
	   // 
	   // _configurationCmbBox
	   // 
	   this._configurationCmbBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
	   this._configurationCmbBox.Location = new System.Drawing.Point(165, 12);
	   this._configurationCmbBox.Name = "_configurationCmbBox";
	   this._configurationCmbBox.Size = new System.Drawing.Size(330, 21);
	   this._configurationCmbBox.Sorted = false;
	   this._configurationCmbBox.TabIndex = 1;
	   this._configurationCmbBox.SelectedIndexChanged += new System.EventHandler(this.SelectedIndexChanged);
	   // 
	   // llpClient
	   // 
	   this.llpClient.MessageReceived += new iNTERFACEWARE.Chameleon.LlpClient.MessageReceivedEvent(this.llpClient_MessageReceived);
	   this.llpClient.Closed += new iNTERFACEWARE.Chameleon.LlpClient.ClosedEvent(this.llpClient_Closed);
	   this.llpClient.ErrorReceived += new iNTERFACEWARE.Chameleon.LlpClient.ErrorReceivedEvent(this.llpClient_ErrorReceived);
	   this.llpClient.DataIgnored += new iNTERFACEWARE.Chameleon.LlpClient.DataIgnoredEvent(this.llpClient_DataIgnored);
	   this.llpClient.Connected += new iNTERFACEWARE.Chameleon.LlpClient.ConnectedEvent(this.llpClient_Connected);
	   // 
	   // _topPanel
	   // 
	   this._topPanel.Controls.AddRange(new System.Windows.Forms.Control[] {
																			   this._configurationLabel,
																			   this._configurationCmbBox});
	   this._topPanel.Dock = System.Windows.Forms.DockStyle.Top;
	   this._topPanel.Name = "_topPanel";
	   this._topPanel.Size = new System.Drawing.Size(542, 45);
	   this._topPanel.TabIndex = 0;
	   // 
	   // ClientForm
	   // 
	   this.AcceptButton = this._sendButton;
	   this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
	   this.ClientSize = new System.Drawing.Size(542, 369);
	   this.Controls.AddRange(new System.Windows.Forms.Control[] {
																	 this._topPanel,
																	 this._middlePanel,
																	 this._bottomPanel});
	   this.Name = "ClientForm";
	   this.Text = "Client";
	   this.Closing += new System.ComponentModel.CancelEventHandler(this.ClientForm_Closing);
	   this.Load += new System.EventHandler(this.ClientForm_Load);
	   this._bottomPanel.ResumeLayout(false);
	   this._middlePanel.ResumeLayout(false);
	   this._topPanel.ResumeLayout(false);
	   this.ResumeLayout(false);

   }
#endregion

	// Client event handlers implementation
	// ------------------------------------
	// This event fires when the Client is disconnected from the Server.
	private void llpClient_Closed(object sender, System.EventArgs e)
	{
		Println("Connection Closed");
		_sendButton.Enabled = false;
		connected = false;		
		_connectDisconnectBtn.Text = "&Connect";
		_portTextBox.Enabled = true;
		_hostTextBox.Enabled = true;
	}

	// This event fires when the Client connects to the Server.
	// The connected event is very important.  One common mistake
	// that developers often make when new to socket programming
	// is to try and invoke the Connect method and then immediately
	// send data via the socket.  This is not possible to do until the
	// socket is actually connected.  Instead,  you should invoke the
	// Connect method and then wait for this event before sending
	// data.
	private void llpClient_Connected(object sender, System.EventArgs e)
	{
		connected = true;
		_sendButton.Enabled = true;
		_connectDisconnectBtn.Text = "&Disconnect";
		_portTextBox.Enabled = false;
		_hostTextBox.Enabled = false;

		Println("Connection established with host\r\n");	
	}

	// This event fires when we receive data that is not using the Lower Layer Protocol (LLP).
	// LLP is the standard protocol for sending HL7 messages, and is used by Chameleon.  See the manual
	// for more details.
	private void llpClient_DataIgnored(object sender, System.EventArgs e, string data)
	{
		Println("Ignored Data received: ");
		Println(engine.MakeMessagePrintable(data));
	}

	// This event fires when we receive a TCP/IP error during connection.
	// This will always terminate the connection, so your real application will 
	// need logic to determine if and when to re-connect to the other host.
	private void llpClient_ErrorReceived(object sender, System.EventArgs e, string description, int code)
	{
		Println(engine.MakeMessagePrintable("Error Received : " + description ));
		_connectDisconnectBtn.Text = "&Connect";
		_portTextBox.Enabled = true;
		_hostTextBox.Enabled = true;
		_sendButton.Enabled = false;
		connected = false;
	}

	// This event fires when a message is received.  Usually the message received 
	// is an ACK message.  If, however, you are implementing a lab query interface 
	// it might be a lab result message.  For this example we are assuming it is an ACK message and
	// just printing it.
	private void llpClient_MessageReceived(object sender, System.EventArgs e, string message)
	{
		Println(engine.MakeMessagePrintable("Received Ack : " + message + "\r\n"));
	}
}

